import collections

import pygame
from pygame.locals import *

from . import const
from . import utils
from . import data


class SpriteSheet(object):
    def __init__(self, sheet_name, tile_size, colorkey=const.COLORKEY):
        self.sheet = data.load_image(sheet_name, colorkey)
        self.tile_size = tile_size 
        self.tile_w, self.tile_h = tile_size
        sheet_rect = self.sheet.get_rect()
        self.rows, self.cols = (
               sheet_rect.h / self.tile_h,
               sheet_rect.w / self.tile_w
               )

    def frame(self, frame_number):
        pos = (
               (frame_number % self.cols) * self.tile_w,
               (frame_number // self.cols) * self.tile_h
               )
        return self.sheet.subsurface((pos, self.tile_size))


class SimpleSprite(pygame.sprite.DirtySprite):
    def __init__(self, scene, start_pos=(0,0), image=None):
        super(SimpleSprite, self).__init__()
        self.scene = scene
        if image == None:
            self.image = pygame.Surface((0,0))
            self.rect = pygame.Rect(start_pos, (1,1))
        else:
            self.image = image
            self.rect = image.get_rect()
            self.rect.topleft = start_pos
        self.setup()

    def setup(self):
        pass

    def set_image(self, new_image):
        self.image = new_image
        new_rect = new_image.get_rect()
        self.rect.size = new_rect.size


class Entity(pygame.sprite.DirtySprite):
    def __init__(self, scene, sheet, start_pos):
        pygame.sprite.DirtySprite.__init__(self)
        self.scene = scene
        self.sheet = sheet
        self.image = self.sheet.frame(0)
        self.rect = self.image.get_rect()
        self.rect.center = start_pos
        self.animation = utils.Animation()
        self.cur_animation = None
        
        self.x = float(self.rect.x)
        self.y = float(self.rect.y)
        self.vx = 0.0
        self.vy = 0.0
        self.last_framenumber = -1
        self.setup()

    def setup(self):
        pass

    def update(self, dt):
        self.update_entity(dt)
        self.animation.update(dt)
        if self.cur_animation is not None:
            self._show_frame(self.animation.play(self.cur_animation))

    def update_entity(self, dt):
        pass

    def _show_frame(self, frame_number):
        if self.last_framenumber != frame_number:
            self.last_framenumber = frame_number
            self.image = self.sheet.frame(frame_number)

    def play(self, animation_name, reset=False):
        self.cur_animation = animation_name
        self.animation.play(animation_name, reset)

    def switch_loop(self, name):
        self.cur_animation = name
        self.animation.switch_loop(name)


class Layer(object):
    def __init__(self):
        self.spritegroups = {}

    def add_group(self, name):
        if name in self.spritegroups:
            raise AttributeError('Layer has already group "%s"' % name) 
        else:
            self.spritegroups[name] = pygame.sprite.RenderUpdates()

    def update(self, dt):
        for group in self.spritegroups.values():
            group.update(dt)

    def add_sprite(self, groupname, *sprites):
        self.spritegroups[groupname].add(*sprites)

    def draw(self, target):
        for group in self.spritegroups.values():
            group.draw(target)

    def rename_group(self, old_name, new_name):
        self.spritegroups[new_name] = self.spritegroups.pop(old_name)

    def group(self, name):
        return self.spritegroups[name]


class Scene(object):
    def __init__(self, engine, size, pos):
        self.engine = engine
        self.size = size
        self.pos = pos
        self.canvas = pygame.Surface(size)
        self.bg_image = pygame.Surface(size)
        self.canvas.set_colorkey(const.COLORKEY) 
        self.bg_image.fill(const.COLORKEY)
        self.layers = collections.OrderedDict({'default': Layer()})
        self.is_paused = False
        self.setup()
        
    def setup(self):
        """Call after __init__.
        Overwrite in child class.
        """
        pass

    def handle_events(self):
        """Call first every frame.
        Overwrite in child class.
        """
        pass

    def update_scene(self, dt):
        """Call second every frame.
        Overwrite in child class.
        """
        pass

    def pause(self):
        self.is_paused = True

    def resume(self):
        self.is_paused = False

    def add_layer(self, layer_name):
        self.layers[layer_name] = Layer()

    def layer(self, layer_name):
        return self.layers[layer_name]

    def update(self, dt):
        if not self.is_paused:
            for name, layer in self.layers.items():
                layer.update(dt)
            self.update_scene(dt)

    def draw(self):
        if not self.is_paused:
            self._clear_canvas()
            for name, layer in self.layers.items():
                layer.draw(self.canvas)
        self.engine.display.blit(self.canvas, self.pos)

    def _clear_canvas(self):
        self.canvas.blit(self.bg_image, (0,0))


class Engine(object):
    def __init__(self, win_size):
        pygame.init()
        pygame.mouse.set_visible(False)
        # flags = pygame.FULLSCREEN
        flags = 0
        self.display = pygame.display.set_mode(win_size, flags)
        self.is_running = True
        self.clock = pygame.time.Clock()
        self.bg_image = pygame.Surface(win_size)
        self.bg_image.fill(const.BG_COLOR)
        self.input = utils.InputController()
        self.rev_mili_sec = 1 / 1000.0
        self.scene_stack = []
        self.setup()

    def setup(self):
        pass

    def add_hud(self, hud):
        self.huds.append(hud)

    def run(self):
        while self.is_running:
            self._handle_events()
            self._update()
            self._render()
        pygame.quit()

    def push_scene(self, scene):
        self.scene_stack.append(scene)

    def pop_scene(self):
        self.scene_stack.pop()
        
    def change_scene(self, scene):
        try:
            self.pop_scene()
        except IndexError:
            pass
        self.push_scene(scene)

    def quit(self):
        self.is_running = False

    def _handle_events(self):
        for e in pygame.event.get():
            if e.type == QUIT:
                self.is_running = False
            elif e.type == KEYDOWN:
                self.input.on_key_down(e.key)
            elif e.type == KEYUP:
                self.input.on_key_up(e.key)
            elif e.type == MOUSEBUTTONDOWN:
                self.input.on_mouse_bt_down(e.pos, e.button)
            elif e.type == MOUSEBUTTONUP:
                self.input.on_mouse_bt_up(e.pos, e.button)
        stack_size = len(self.scene_stack)
        if stack_size > 0:
            self.scene_stack[stack_size-1].handle_events()

    def _update(self):
        self.input.clear_key_events()
        dt = self.clock.tick(const.FPS) * self.rev_mili_sec
        stack_size = len(self.scene_stack)
        if stack_size > 0:
            self.scene_stack[stack_size-1].update(dt)

    def _render(self):
        self._clear_screen()
        if len (self.scene_stack) > 0:
            for scene in self.scene_stack:
                scene.draw()
        pygame.display.flip()

    def _clear_screen(self):
        self.display.blit(self.bg_image, (0,0))


class MenuItem(pygame.sprite.DirtySprite):
    def __init__(self, font, text, pos, color=const.MENU_TEXT_COLOR):
        super(MenuItem, self).__init__()
        self.color = color
        self.font = font
        self.text = text
        self.pos = pos
        self.render_text(color)

    def render_text(self, color):
        self.image = self.font.render(self.text, False, color)
        self.rect = self.image.get_rect()
        self.rect.center = self.pos


class Menu(Scene):
    def __init__(self, engine, size, pos, callback_select):
        super(Menu, self).__init__(engine, size, pos)
        self.callback = callback_select

    def setup(self):
        self.layer('default').add_group('menu')
        filename = data.filepath('fonts/manaspc.ttf')
        self.font = pygame.font.Font(filename, 18)
        self.titlefont = pygame.font.Font(filename, 30)
        self.menu_items = []
        self.cur_selection = 0
        self.has_title = False
        self.has_subtitle = False
        self.setup_menu()

    def setup_menu(self):
        pass

    def add_menu_title(self, text):
        self.menu_title = MenuItem(
                self.titlefont, text, (self.size[0]/2, 20),
                const.MENU_TEXT_COLOR_SEL)
        self.layer('default').add_sprite('menu', self.menu_title)
        self.has_title = True

    def add_subtitle(self, text):
        self.menu_subtitle = MenuItem(
                self.font, text, (self.size[0]/2, 50),
                const.MENU_TEXT_COLOR_SEL)
        self.layer('default').add_sprite('menu', self.menu_subtitle)
        self.has_subtitle = True

    def add_menu_item(self, text):
        index = len(self.menu_items)
        color = const.MENU_TEXT_COLOR
        if index == self.cur_selection:
            color = const.MENU_TEXT_COLOR_SEL
        x = self.size[0]/2
        y = len(self.menu_items) * 25 + 150
        if self.has_subtitle:
            y += 50
        item = MenuItem(self.font, text, (x, y), color)
        self.menu_items.append(item)
        self.layer('default').add_sprite('menu', item)

    def handle_events(self):
        if self.engine.input.pressed(K_UP):
            self._on_item_up()
        elif self.engine.input.pressed(K_DOWN):
            self._on_item_down()
        elif self.engine.input.pressed(K_RETURN):
            self.callback(self.cur_selection)

    def _on_item_up(self):
        self.menu_items[self.cur_selection].render_text(const.MENU_TEXT_COLOR)
        self.cur_selection -= 1
        if self.cur_selection < 0:
            self.cur_selection = len(self.menu_items) -1
        self.menu_items[self.cur_selection].render_text(const.MENU_TEXT_COLOR_SEL)

    def _on_item_down(self):
        self.menu_items[self.cur_selection].render_text(const.MENU_TEXT_COLOR)
        self.cur_selection += 1
        if self.cur_selection > len(self.menu_items) -1:
            self.cur_selection = 0
        self.menu_items[self.cur_selection].render_text(const.MENU_TEXT_COLOR_SEL)

