'''Game main module.

Contains the entry point used by the run_game.py script.

Feel free to put all your game code here, or in other modules in this "gamelib"
package.
'''
import random
import math

import pygame
from pygame.locals import *

# import data
from . import data
from . import const
from .engine import Engine, Layer, Scene, SpriteSheet, Entity
from .engine import Menu, SimpleSprite
from . import utils
from .utils import Animation

class Deflector(Entity):
    def setup(self):
        self.angle_deg = 270.0
        #NOTE:
        """this is pointing North.
        """
        self.angle_speed = 0.0
        self.angle_acc = 700
        self.max_angle_speed = 160
        self.arc_centerx = const.DEFL_X
        self.arc_centery = const.DEFL_Y
        self.radius_x = 130
        self.radius_y = 100
        self.move_dir = '0'
        self.paddle_modes = ['normal', 'wave']
        self.paddle_mode_index = 0
        self.friction = 20
        self.has_rev_dir = False 

        self.animation.add_loop('normal-E', [0], 12)
        self.animation.add_loop('normal-ENE', [1], 12)
        self.animation.add_loop('normal-NE', [2], 12)
        self.animation.add_loop('normal-N', [3], 12)
        self.animation.add_loop('normal-NW', [4], 12)
        self.animation.add_loop('normal-WNW', [5], 12)
        self.animation.add_loop('normal-W', [6], 12)

        self.animation.add_loop('wave-E', [7], 12)
        self.animation.add_loop('wave-ENE', [8], 12)
        self.animation.add_loop('wave-NE', [9], 12)
        self.animation.add_loop('wave-N', [10], 12)
        self.animation.add_loop('wave-NW', [11], 12)
        self.animation.add_loop('wave-WNW', [12], 12)
        self.animation.add_loop('wave-W', [13], 12)
        self.play('normal-N')

    def change_mode(self, change):
        self.paddle_mode_index += change
        if self.paddle_mode_index < 0:
            self.paddle_mode_index = len(self.paddle_modes)-1
        elif self.paddle_mode_index > len(self.paddle_modes)-1:
            self.paddle_mode_index = 0

    def _get_normal(self):
        #NOTE:
        """The polar coords of pg Vector2 point East at 0 degrees.
        """
        phi = self.angle_deg + 180
        vector = pygame.math.Vector2()
        vector.from_polar((1, phi))
        return (vector)

    def move_cw(self):
        """Call during event handling.
        """
        self.move_dir = 'cw'

    def move_ccw(self):
        self.move_dir = 'ccw'

    def _move_cw(self, dt):
        """Call during update.
        I think it's kind of ugly.
        """
        self.angle_speed += self.angle_acc * dt
        if self.angle_speed >= self.max_angle_speed:
            self.angle_speed = self.max_angle_speed

    def _move_ccw(self, dt):
        self.angle_speed -= self.angle_acc * dt
        if self.angle_speed <= -self.max_angle_speed:
            self.angle_speed = -self.max_angle_speed

    def change_direction(self):
        self.has_rev_dir = not self.has_rev_dir
    
    def update_entity(self, dt):
        if self.move_dir == 'cw':
            if self.has_rev_dir:
                self._move_ccw(dt)
            else:
                self._move_cw(dt)
        elif self.move_dir == 'ccw':
            if self.has_rev_dir:
                self._move_cw(dt)
            else:
                self._move_ccw(dt)
        else:
            self.angle_speed *= 0.7

        self._set_position(dt)
        self._rotate_sprite(self.angle_deg)
        self._check_collision()

    def _set_position(self, dt):
        self.angle_deg += self.angle_speed * dt
        if self.angle_deg < 180:
            self.angle_deg = 180
            self.angle_speed = 0
        elif self.angle_deg > 360:
            self.angle_deg = 360
            self.angle_speed = 0
        self.x, self.y = utils.point_on_ellipse(
                math.radians(self.angle_deg),
                self.radius_x,
                self.radius_y,
                self.arc_centerx,
                self.arc_centery
                )
        self.move_dir = '0'
        self.rect.center = (self.x, self.y)

    def _rotate_sprite(self, a):
        prefix = self.paddle_modes[self.paddle_mode_index]
        if a >= 180 and a <= 198:
            self.play('%s-E' % prefix)
        elif a >= 198 and a <= 230:
            self.play('%s-ENE' % prefix)
        elif a >= 230 and a <= 255:
            self.play('%s-NE' % prefix)
        elif a >= 255 and a <= 285:
            self.play('%s-N' % prefix)
        elif a >= 285 and a <= 310:
            self.play('%s-NW' % prefix)
        elif a >= 310 and a <= 342:
            self.play('%s-WNW' % prefix)
        elif a >= 342 and a <= 360:
            self.play('%s-W' % prefix)

    def _check_collision(self):
        group = self.scene.layer('enemies').group('pills')
        collision_list = pygame.sprite.spritecollide(self, group, False)
        for pill in collision_list:
            if self.paddle_modes[self.paddle_mode_index] == 'normal':
                pill.reflect(self._get_normal())
        group = self.scene.layer('enemies').group('waves')
        collision_list = pygame.sprite.spritecollide(self, group, False)
        for wave in collision_list:
            if self.paddle_modes[self.paddle_mode_index] == 'wave':
                wave.reflect(self._get_normal())


class Intro(Scene):
    def __init__(self, engine):
        super(Intro, self).__init__(engine, const.WIN_SIZE, (0,0))

    def setup(self):
        self.cur_paragraph = 0
        self.layer('default').add_group('intro')
        self.splash_screen = data.load_image(
                'splash_screen.png',
                const.COLORKEY
                )
        self.duration = 6
        self.cur_time = self.duration
        self._render_paragraph()
        self.has_shown_splash = False
        self.do_skip = False

    def update_scene(self, dt):
        self.cur_time -= dt
        if self.cur_time <= 0 or self.do_skip:
            self.cur_paragraph += 1
            self.do_skip = False
            if self.cur_paragraph >= len(utils.intro):
                if self.has_shown_splash:
                    self.engine.on_intro_ready()
                else:
                    self.show_splash()
            else:
                self.cur_time = self.duration
                self._render_paragraph()

    def show_splash(self):
        self.layer('default').group('intro').empty()
        self.bg_image.blit(self.splash_screen, (0,0))
        self.cur_time = 5
        if self.do_skip:
            self.cur_time = 0
        self.has_shown_splash = True

    def _render_paragraph(self):
        self.layer('default').group('intro').empty()
        line_1 = SimpleSprite(
                self, (0,0),
                utils.text_controller.render_text(
                    utils.intro[self.cur_paragraph][0],
                    'yoster'
                    )
                )
        line_2 = SimpleSprite(
                self, (0,0),
                utils.text_controller.render_text(
                    utils.intro[self.cur_paragraph][1],
                    'yoster'
                    )
                )
        line_1.rect.center = ((320, 180))
        line_2.rect.center = ((320, 220))
        self.layer('default').add_sprite('intro', line_1)
        self.layer('default').add_sprite('intro', line_2)

    def handle_events(self):
        if self.engine.input.pressed(K_ESCAPE):
            self._skip()

    def _skip(self):
        self.do_skip = True
        # self.show_splash(True)

class GameOverMenu(Menu):
    def __init__(self, engine):
        super(GameOverMenu, self).__init__(
                engine,
                const.MENU_SIZE, const.MENU_POS,
                engine.on_game_over_menu_click
                )

    def setup_menu(self):
        self.add_menu_title('Game Over')
        self.add_subtitle('You got %d points' % self.engine.points)
        self.add_menu_item('new game')
        self.add_menu_item('quit')


class StartMenu(Menu):
    def __init__(self, engine):
        super(StartMenu, self).__init__(
                engine,
                const.MENU_SIZE, const.MENU_POS,
                engine.on_start_menu_click,
                )

    def setup_menu(self):
        self.add_menu_title('Welcome!')
        self.add_subtitle('Pyweek 23! it was fun :)')
        self.add_menu_item('instructions')
        self.add_menu_item('start game')
        self.add_menu_item('quit')


class MainMenu(Menu):
    def __init__(self, engine, callback, is_fullscreen):
        self.is_fullscreen = is_fullscreen
        super(MainMenu, self).__init__(
                engine,
                const.MENU_SIZE, const.MENU_POS,
                callback
                )
        
    def setup_menu(self):
        if self.engine.language == 'english':
            self.add_menu_title('Pause')
            self.add_menu_item('resume')
            self.add_menu_item('quit')
        else:
            self.add_menu_title('Unterbrechung')
            self.add_menu_item('weiter')
            self.add_menu_item('beenden')

        if self.is_fullscreen:
            if self.engine.language == 'english':
                self.add_menu_item('windowed')
            else:
                self.add_menu_item('im Fenster')
        else: 
            if self.engine.language == 'english':
                self.add_menu_item('fullscreen')
            else:
                self.add_menu_item('Vollbild')


class OnScreenDisplay(SimpleSprite):
    def __init__(self, scene, message, callback_ready, param, time):
        # NOTE: Would be nice to select fade direction
        """ Maybe for the future.
        """
        image = pygame.Surface((400, 50))
        image.fill(const.COLORKEY)
        image.set_colorkey(const.COLORKEY)
        super(OnScreenDisplay, self).__init__(scene, (20, 30), image)
        self.callback = callback_ready
        self.callback_parameter = param
        self.duration = time
        self.cur_time = 0.0
        self.message_text = utils.text_controller.render_text(
                message, 'mana', (200, 255, 255)
                )
        self.image.blit(self.message_text, (0,0))
        self.progress_bar = pygame.Rect((30,36), (100,5))

    def setup(self):
        pass

    def update(self, dt):
        self._clear()
        self.cur_time += dt
        if self.cur_time >= self.duration:
            self.callback(self.callback_parameter)
            self.kill()
        self._render()

    def _clear(self):
        self.image.fill(const.COLORKEY)

    def _render(self):
        self.progress_bar.w = utils.lin_tween(
                self.cur_time, 100, -100, self.duration
                )
        self.image.blit(self.message_text, (0,0))
        pygame.draw.rect(self.image, (200, 255, 255), self.progress_bar)


class SlidePlane(SimpleSprite):
    def __init__(self, scene, bg_color, callback_ready, time):
        # NOTE: Would be nice to select fade direction
        """ Maybe for the future.
        """
        image = pygame.Surface(const.WIN_SIZE)
        image.fill(bg_color)
        super(SlidePlane, self).__init__(scene, (0, -scene.size[1]), image)
        self.callback = callback_ready
        self.duration = time

    def setup(self):
        # self.duration = .3
        self.start_value = -self.scene.size[1]
        self.cur_time = 0.0
        self.change = self.scene.size[1]
        self.is_finished = False

    def update(self, dt):
        if not self.is_finished:
            self.cur_time += dt
            self.y = utils.quad_in(
                    self.cur_time, self.start_value, self.change,
                    self.duration
                    )
            self.rect.y = int(self.y)

            if self.cur_time >= self.duration:
                self.is_finished = True
                self.rect.y = 0
                self.callback()


class Instructions(Scene):
    def __init__(self, engine, callback):
        super(Instructions, self).__init__(engine, const.WIN_SIZE, (0,0))
        image = data.load_image('instructions.png', const.COLORKEY)
        self.bg_image.blit(image, (0,0))
        self.callback = callback

    def handle_events(self):
        if self.engine.input.pressed(K_ESCAPE):
            self.callback()


class MessageScreen(Scene):
    def __init__(self, engine, title, note):
        self.note_text = note
        self.title_text = title
        super(MessageScreen, self).__init__(engine, const.WIN_SIZE, (0,0))

    def setup(self):
        self.is_finished = False
        self.duration = 2
        self.layer('default').add_group('main')
        
        self.header = SimpleSprite(
                self, (0, 0),
                utils.text_controller.render_title(
                    "OH NO!", (255, 30, 10)
                    )
                )
        self.title = SimpleSprite(
                self, (0, 0),
                utils.text_controller.render_title(
                    self.title_text
                    )
                )

        self.note = SimpleSprite(
                self, (0, 0),
                utils.text_controller.render_text(
                    self.note_text,
                    ('yoster')
                    )
                )
        self.header.rect.center = ((const.WIN_SIZE[0]/2, 100))
        self.title.rect.center = ((const.WIN_SIZE[0]/2, 200))
        self.note.rect.center = ((const.WIN_SIZE[0]/2, 300))

        self.layer('default').add_sprite('main', self.note)
        self.layer('default').add_sprite('main', self.header)
        self.layer('default').add_sprite('main', self.title)

    def update(self, dt):
        if not self.is_finished:
            self.duration -= dt
            self.header.rect.x += random.randint(-2, 2)
            if self.duration <= 0:
                self.is_finished = True
                self.engine.on_message_finish()


class Transition(Scene):
    def __init__(self, engine, size, pos, duration, next_scene):
        self.duration = duration
        self.next_scene = next_scene
        super(Transition, self).__init__(engine, size, pos)

    def setup(self):
        plane = SlidePlane(
                self, (4,0,0), self.on_plane_slide_in, self.duration
                )

        self.layer('default').add_group('plane')
        self.layer('default').add_sprite('plane', plane)

        self.canvas.set_alpha(250)
        self.slide_finished = False

    def on_plane_slide_in(self):
        self.slide_finished = True

    def update_scene(self, dt):
        if self.slide_finished:
            self.engine.push_scene(self.next_scene)


class BrainPart(Entity):
    def __init__(self, scene, sheet, start_pos, controller, name): 
        self.name = name
        self.controller = controller
        self.colorscheme = 'multicolor'
        super(BrainPart, self).__init__(scene, sheet, start_pos)

    def setup(self):
        self.animation.add_oneshot(
                'hitting',
                [1,2,1,2,1,2,2,3,3,4,4,5,5,6,6,7,7,8,8],
                24,
                self.on_hit_animation_ready
                )
        self.animation.add_loop('dead', [9], 12)
        self.animation.add_loop('idle', [0], 12)
        self.animation.add_loop('idle-mono', [10], 12)
        self.animation.add_loop('dead-mono', [19], 12)
        self.animation.add_oneshot(
                'hitting-mono',
                [10,11,10,11,10,11,11,12,12,13,13,14,14,15,15,16,16,17,17],
                24,
                self.on_hit_animation_ready
                )
        self.play('idle', True)
        self.is_dead = False

    def on_hit_animation_ready(self):
        if self.colorscheme == 'multicolor':
            self.play('idle')
        else:
            self.play('idle-mono')
        self.remove(self.scene.layer('brain_parts').group('main'))
        self.add(self.scene.layer('brain_bottom').group('main'))
        self.controller.on_hit(self.name)

    def on_collision(self):
        if self.colorscheme == 'multicolor':
            self.play('hitting', True)
        else:
            self.play('hitting-mono', True)
        # self.is_dead = True

    def update_entity(self, dt):
        pill_group = self.scene.layer('enemies').group('pills')
        wave_group = self.scene.layer('enemies').group('waves')
        if not self.is_dead:
            colliding_pills = pygame.sprite.spritecollide(
                    self, pill_group, True
                    )
            colliding_waves = pygame.sprite.spritecollide(
                    self, wave_group, True
                    )
            if colliding_pills or colliding_waves:
                self.on_collision()

    def set_monochrome(self):
        self.play('idle-mono')
        self.colorscheme = 'monochrome'

    def set_multicolor(self):
        self.play('idle')
        self.colorscheme = 'multicolor'


class Brain(Entity):
    def setup(self):
        self.animation.add_loop('multicolor', [0], 12)
        self.animation.add_loop('monochrome', [1], 12)
        self.play('multicolor')

    def set_monochrome(self):
        self.play('monochrome')

    def set_multicolor(self):
        self.play('multicolor')


class BrainController(object):
    #NOTE: This Thing is maybe too much.
    """Either make it completley independent from level or get rid
    of it.
    """
    def __init__(self, scene):
        self.game = scene.engine
        self.scene = scene
        self.flag_frontal = False
        self.flag_motor = False
        self.flag_parietal = False
        self.flag_occipital = False
        self.flag_temporal = False
        
        frontal_sheet = SpriteSheet('ani_hit_frontal.png',const.FRONTAL_SIZE) 
        motor_sheet = SpriteSheet('ani_hit_motor.png',const.MOTOR_SIZE) 
        parietal_sheet = SpriteSheet('ani_hit_parietal.png',const.PARIETAL_SIZE) 
        occipital_sheet = SpriteSheet('ani_hit_occipital.png',const.OCCIPITAL_SIZE) 
        temporal_sheet = SpriteSheet('ani_hit_temporal.png',const.TEMPORAL_SIZE) 
        brain_sheet = SpriteSheet('brain.png', const.BRAIN_SIZE)

        self.brain = Brain(self, brain_sheet, const.BRAIN_POS)
        self.frontal = BrainPart(
                self.scene, frontal_sheet, const.FRONTAL_POS, self, 'frontal'
                )
        self.motor = BrainPart(
                self.scene, motor_sheet, const.MOTOR_POS, self, 'motor'
                )
        self.parietal = BrainPart(
                self.scene, parietal_sheet, const.PARIETAL_POS, self, 'parietal'
                )
        self.occipital = BrainPart(
                self.scene, occipital_sheet, const.OCCIPITAL_POS, self, 'occipital'
                )
        self.temporal = BrainPart(
                self.scene, temporal_sheet, const.TEMPORAL_POS, self, 'temporal'
                )

        self.layer_bottom = self.scene.layer('brain_bottom')
        self.layer_top = self.scene.layer('brain_parts')
        self.layer_default = self.scene.layer('default')

        self.layer_bottom.add_sprite('main', self.frontal, self.motor,
                                     self.parietal, self.occipital,
                                     self.temporal
                                     )
        self.layer_default.add_sprite('brain', self.brain)

    def set_monochrome(self):
        self.brain.set_monochrome()
        self.motor.set_monochrome()
        self.parietal.set_monochrome()
        self.frontal.set_monochrome()
        self.occipital.set_monochrome()
        self.temporal.set_monochrome()

    def set_multicolor(self):
        self.brain.set_multicolor()
        self.motor.set_multicolor()
        self.parietal.set_multicolor()
        self.frontal.set_multicolor()
        self.occipital.set_multicolor()
        self.temporal.set_multicolor()

    def on_hit(self, brain_part):
        self.game.hit_brain_part(brain_part)

        
class Pill(Entity):
    def __init__(self, scene, sheet, start_pos, colorscheme='multicolor'):
        self.colorscheme = colorscheme
        super(Pill, self).__init__(scene, sheet, start_pos)
    
    def setup(self):
        self.animation.add_loop('spin', [0,1,2,3,4,5,6,7], 12)
        self.animation.add_loop('spin-mono', [8,9,10,11,12,13,14,15], 12)
        target_x, target_y = const.BRAIN_POS
        self.v = pygame.math.Vector2(
                target_x-self.x,
                target_y-self.y
                )
        self.v.normalize_ip()
        if self.colorscheme == 'multicolor':
            self.play('spin')
        else:
            self.play('spin-mono')
        self.not_visible_yet = False
        self.scene_rect = pygame.Rect((0,0), self.scene.size)
        self.speed = 150
        if not self.scene_rect.contains(self.rect):
            self.not_visible_yet = True
        self.has_reflected = False
        self.rect.size = (6, 6)

    def reflect(self, normal):
        if not self.has_reflected:
            self.v.reflect_ip(normal)
            self.has_reflected = True
            self.scene.engine.on_reflect_enemy('pill')

    def update_entity(self, dt):
        self.x += self.v.x * dt * self.speed
        self.y += self.v.y * dt * self.speed
        self.rect.x = int(self.x)
        self.rect.y = int(self.y)
        if not self.scene_rect.contains(self.rect):
            if not self.not_visible_yet:
                self.kill()
        else:
            self.not_visible_yet = False

    def set_monochrome(self):
        self.colorscheme = 'monochrome'
        self.switch_loop('spin-mono')

    def set_multicolor(self):
        pass
        
        
class Wave(Entity):
    def __init__(self, scene, sheet, start_pos, colorscheme='multicolor'):
        self.colorscheme = colorscheme
        super(Wave, self).__init__(scene, sheet, start_pos)
    
    def setup(self):
        self.animation.add_loop('spin', [0,1,2,3,4], 12)
        self.animation.add_loop('spin-mono', [5,6,7,8,9], 12)
        target_x, target_y = const.BRAIN_POS
        self.v = pygame.math.Vector2(
                target_x-self.x,
                target_y-self.y
                )
        self.v.normalize_ip()
        if self.colorscheme == 'multicolor':
            self.play('spin')
        else:
            self.play('spin-mono')
        self.not_visible_yet = False
        self.scene_rect = pygame.Rect((0,0), self.scene.size)
        self.speed = 150
        if not self.scene_rect.contains(self.rect):
            self.not_visible_yet = True
        self.has_reflected = False
        self.rect.size = (6, 6)

    def reflect(self, normal):
        if not self.has_reflected:
            self.v.reflect_ip(normal)
            self.has_reflected = True
            self.scene.engine.on_reflect_enemy('wave')

    def update_entity(self, dt):
        self.x += self.v.x * dt * self.speed
        self.y += self.v.y * dt * self.speed
        self.rect.x = int(self.x)
        self.rect.y = int(self.y)
        if not self.scene_rect.contains(self.rect):
            if not self.not_visible_yet:
                self.kill()
        else:
            self.not_visible_yet = False

    def set_monochrome(self):
        self.colorscheme = 'monochrome'
        self.switch_loop('spin-mono')

    def set_multicolor(self):
        pass
        

class SaneMeter(Entity):
    def setup(self):
        self.last_index = 5
        self._show_frame(0)

    def set_sanity(self, value):
        if value > 1:
            value = int(value/2)
        self._show_frame(self.last_index - value)

    def set_monochrome(self):
        self.last_index = 11

    def set_multicolor(self):
        self.last_index = 5


class PointCounter(SimpleSprite):
    def __init__(self, scene, start_pos, points=0):
        super(PointCounter, self).__init__(scene, start_pos)
        self.render_points(points)
      
    def render_points(self, points, color=const.TEXT_COL):
        point_text = "%06d" % points
        self.set_image(
                utils.text_controller.render_text(
                    point_text, 'yoster', color
                    )
                )


class HUD(Scene):
    def setup(self):
        self.bg_image.fill((10,10,10)) 
        self.colorscheme = 'multicolor'
        # self.text_controller = TextController(self)
        meter_sheet = SpriteSheet('sane_meter_final.png', (const.METER_SIZE))

        self.layer('default').add_group('main')
        self.sane_meter = SaneMeter(self, meter_sheet, const.METER_POS)
        self.point_counter = PointCounter(self, const.POINTS_POS)
        self.title_sane_meter = SimpleSprite(
                self, (5, 5), utils.text_controller.render_text(
                    'Sanity:', 'mana', (200, 200, 200)
                    )
                )
        self.title_point_counter = SimpleSprite(
                self, (500, 5), utils.text_controller.render_text(
                    'Points:', 'mana', (200, 200, 200)
                    )
                )
        self.layer('default').add_sprite('main', self.sane_meter)
        self.layer('default').add_sprite('main', self.point_counter)
        self.layer('default').add_sprite('main', self.title_sane_meter)
        self.layer('default').add_sprite('main', self.title_point_counter)

    def set_sanity(self, sanity):
        self.sane_meter.set_sanity(sanity)

    def set_points(self, points):
        self.point_counter.render_points(points)
        if self.colorscheme == 'monochrome':
            self.point_counter.render_points(points,(0,69,69))
        else:
            self.point_counter.render_points(points)

    def set_monochrome(self):
        self.colorscheme = 'monochrome'
        self.bg_image.fill((0, 20, 20))
        self.sane_meter.set_monochrome()
        self.set_sanity(self.engine.sanity)
        self.title_point_counter.set_image(
                utils.text_controller.render_text(
                    'Points:', 'mana', (0, 255,255)
                    )
                )
        self.title_sane_meter.set_image(
                utils.text_controller.render_text(
                    'Sanity:', 'mana', (0, 255,255)
                    )
                )

    def set_multicolor(self):
        self.colorscheme = 'multicolor'
        self.bg_image.fill((10,10,10))
        self.sane_meter.set_multicolor()
        self.sane_meter.set_sanity(self.engine.sanity)
        self.title_point_counter.set_image(
                utils.text_controller.render_text(
                    'Points:', 'mana', (200,200,200)
                    )
                )
        self.title_sane_meter.set_image(
                utils.text_controller.render_text(
                    'Sanity:', 'mana', (200,200,200)
                    )
                )

    def set_gibberish(self):
        self.title_point_counter.set_image(
                utils.text_controller.render_text(
                    'Punkte:', 'mana', (200,200,200)
                    )
                )
        self.title_sane_meter.set_image(
                utils.text_controller.render_text(
                    'Verstand:', 'mana', (200,200,200)
                    )
                )
       
    def set_english(self):
        self.title_point_counter.set_image(
                utils.text_controller.render_text(
                    'Points:', 'mana', (200,200,200)
                    )
                )
        self.title_sane_meter.set_image(
                utils.text_controller.render_text(
                    'Sanity:', 'mana', (200,200,200)
                    )
                )



class Level_1(Scene):
    def setup(self):
        self.colorscheme = 'multicolor'
        self.spawn_time = 2

        self.spawn_timer = self.spawn_time
        deflector_sheet = SpriteSheet('ani_deflector_2x.png', (46, 46))
        # brain_sheet = SpriteSheet('brain.png', const.BRAIN_SIZE)
        # self.pill_sheet = SpriteSheet('ani_pill.png',const.PILL_SIZE) 
        self.pill_sheet = SpriteSheet('ani_pill_2x.png',(32, 32)) 
        self.wave_sheet = SpriteSheet('ani_wave_2x.png', (32, 32))

        # self.wave = Wave(self, wave_sheet, (20,20))
        # self.brain = Brain(self, brain_sheet, const.BRAIN_POS)
        self.deflector = Deflector(self, deflector_sheet, (0,0))
        self.engine.register_deflector(self.deflector)

        self.add_layer('brain_bottom')
        self.add_layer('brain_parts')
        self.add_layer('enemies')
        self.add_layer('messages')
        
        self.layer('brain_bottom').add_group('main')
        self.layer('brain_parts').add_group('main')
        self.layer('brain_parts').add_group('deflector')
        self.layer('enemies').add_group('pills')
        self.layer('enemies').add_group('waves')
        self.layer('default').add_group('brain')
        self.layer('messages').add_group('main')

        # self.layer('default').add_sprite('brain', self.brain)
        self.layer('brain_parts').add_sprite('deflector', self.deflector)

        self.brain_controller = BrainController(self)
        self.enemies_spawned = 0

    def set_monochrome(self):
        self.colorscheme = 'monochrome'
        self.brain_controller.set_monochrome()
        for sprite in self.layer('enemies').group('pills').sprites():
            sprite.set_monochrome()
        for sprite in self.layer('enemies').group('waves').sprites():
            sprite.set_monochrome()

    def set_multicolor(self):
        self.colorscheme = 'multicolor'
        self.brain_controller.set_multicolor()
        for sprite in self.layer('enemies').group('pills').sprites():
            sprite.set_multicolor()
        for sprite in self.layer('enemies').group('waves').sprites():
            sprite.set_multicolor()

    def on_feedback_ready(self):
        pass
        # print ('message allready gone!')

    def handle_events(self):
        engine = self.engine

        if engine.input.check(K_LEFT):
            self.deflector.move_ccw()
        
        elif engine.input.check(K_RIGHT):
            self.deflector.move_cw()

        if engine.input.pressed(K_UP):
            self.deflector.change_mode(-1)

        if engine.input.pressed(K_DOWN):
            self.deflector.change_mode(1)

        elif engine.input.pressed(K_ESCAPE):
            menu = MainMenu(
                    engine, engine.on_menu_item_click,
                    self.engine.is_fullscreen
                    )
            engine.push_scene(
                    Transition(
                        self.engine, const.WIN_SIZE, (0,0),
                        const.FADE_TIME, menu
                        )
                    )

    def update_scene(self, dt):
        self.spawn_timer -= dt
        if self.spawn_timer <= 0:
            self.spawn_timer = self.spawn_time
            self._spawn_enemy()

    def _spawn_enemy(self):
        self.enemies_spawned += 1
        if self.enemies_spawned > 30:
            self.spawn_time = 1.8
        if self.enemies_spawned > 50:
            self.spawn_time = 1.5
        if self.enemies_spawned > 100:
            self.spawn_time = 1
        if self.enemies_spawned > 150:
            self.spawn_time = 0.8
        if self.enemies_spawned > 180:
            self.spawn_time = 0.6
        if self.enemies_spawned > 200:
            self.spawn_time = 0.4

        pos = utils.random_on_arc(340, 300, 300)
        if (random.random() > 0.3):
            group = self.layer('enemies').group('pills')
            group.add(Pill(self, self.pill_sheet, pos, self.colorscheme))
        else:
            group = self.layer('enemies').group('waves')
            group.add(Wave(self, self.wave_sheet, pos, self.colorscheme))


class Game(Engine):
    def __init__(self):
        Engine.__init__(self, const.WIN_SIZE)

    def setup(self):
        self.points = 0
        self.sanity = 11
        self.parts_hit = []
        self.deflector = None
        self.colorscheme = 'multicolor'
        self.point_factor = 1.0
        self.language = 'english'
        self.is_game_over = False

        self.head_up = (HUD(self, const.HUD_SIZE, const.HUD_POS))
        self.current_level = Level_1(self, const.LEVEL_SIZE, const.LEVEL_POS)
        self.intro = Intro(self)
        self.push_scene(self.intro)

        self.is_fullscreen = False
        self.effect_is_running = False

    def register_deflector(self, deflector):
        self.deflector = deflector

    def on_intro_ready(self):
        # self.pop_scene()
        start_menu = StartMenu(self)
        transition = Transition(
                self, const.WIN_SIZE, (0,0), .4, start_menu
                )
        self.change_scene(transition)
        # self.pop_scene()
        # self.push_scene(self.head_up)
        # self.push_scene(self.current_level) 

    def on_menu_item_click(self, item):
        if (item == const.MAIN_MENU_QUIT):
            self.quit()
        elif item == const.MAIN_MENU_RESUME:
            self.pop_scene()
            self.pop_scene()
        elif item == const.MAIN_MENU_FULLSCREEN:
            self.is_fullscreen = not self.is_fullscreen
            pygame.display.toggle_fullscreen()
            self.pop_scene()
            self.push_scene(MainMenu(self, self.on_menu_item_click,
               self.is_fullscreen))

    def on_start_menu_click(self, item):
        if item == 0:
            self.pop_scene()
            self.change_scene(
                    Instructions(
                        self,
                        self.on_intro_ready
                        )
                    )
        if item == 1:
            self.pop_scene()
            self.pop_scene()
            self.push_scene(self.head_up)
            self.push_scene(self.current_level) 
        elif item == 2:
            self.quit()

    def on_game_over_menu_click(self, item):
        if item == const.MAIN_MENU_QUIT:
            self.quit()
        elif item == const.MENU_NEW_GAME:
            self.reset_game()

    def on_reflect_enemy(self, enemy):
        if enemy == 'pill':
            self.points += int(self.point_factor * 20)
        elif enemy == 'wave':
            self.points += int(self.point_factor * 5)
        self.head_up.set_points(self.points)

    def hit_brain_part(self, part):
        self.sanity -=1
        if self.sanity <= 0:
            self.sanity = 0
            self.set_game_over()
        self.head_up.set_sanity(self.sanity)
        if not self.effect_is_running:
            self.effect_is_running = True
            effect = random.randint(1,4)
            if effect == 1:
                self.set_direction_change()
            elif effect == 2:
                self.set_monochrome()
            elif effect == 3:
                self.set_gibberish()
            elif effect == 4:
                self.set_half_points()

    def show_game_over_screen(self):
        game_over_screen = GameOverMenu(self)
        self.push_scene(Transition(self,const.WIN_SIZE, (0,0), 1,
            game_over_screen))

    def set_game_over(self):
        self.is_game_over = True
        self.show_game_over_screen()

    def reset_game(self):
        self.points = 0
        self.is_game_over = False
        self.language = 'english'
        self.sanity = 11
        self.parts_hit = []
        self.pop_scene()
        self.pop_scene()
        self.current_level = Level_1(self, const.LEVEL_SIZE, const.LEVEL_POS)
        self.head_up.set_sanity(self.sanity)
        self.head_up.set_multicolor()
        self.head_up.set_points(0)
        self.set_multicolor()
        self.change_scene(self.current_level)

    def set_multicolor(self):
        self.colorscheme = 'multicolor'
        self.current_level.set_multicolor()
        self.bg_image.fill(const.BG_COLOR)
        self.head_up.set_multicolor()

    def set_monochrome(self):
        self.colorscheme = 'monochrome'
        self.current_level.set_monochrome()
        self.head_up.set_monochrome()
        self.bg_image.fill((0,0,0))
        self.show_message('occipital')

    def set_half_points(self):
        self.show_message('frontal')
        self.point_factor = 0.5

    def set_direction_change(self):
        self.show_message('motor')
        self.deflector.change_direction()

    def set_tiny_enemies(self):
        self.show_message('parietal')
        self.current_level.set_tiny_enemies()

    def set_gibberish(self):
        self.language = 'gibberish'
        self.head_up.set_gibberish()
        self.show_message('temporal')

    def set_english(self):
        self.language = 'english'
        self.head_up.set_english()

    def show_message(self, brain_part):
        seconds = 10 
        if brain_part == 'frontal':
            seconds = 20
        message = OnScreenDisplay(
                self,
                utils.messages[brain_part][self.language][1],
                self.on_feedback_ready,
                brain_part,
                seconds
                )
        self.current_level.layer('messages').group('main').add(message)

    def on_feedback_ready(self, brain_part):
        self.effect_is_running = False
        if brain_part == 'motor':
            self.deflector.change_direction()
        if brain_part == 'occipital':
            self.set_multicolor()
        if brain_part == 'frontal':
            self.point_factor = 1.0
        if brain_part == 'temporal':
            self.set_english()


def main():
    game = Game()
    game.run()
    pygame.quit()
