import random
import math

import pygame
from pygame.locals import K_UP, K_DOWN, K_LEFT, K_RIGHT, K_ESCAPE, K_w
from .const import TEXT_COL

from . import data

"""This module should provide Helper classes.
Maybe they are not so close to pygame and therefor live in here.
"""
intro = [[
            "Do you feel like you're being used",
            "as a guinea pig? Who does that to you?"
        ],
        [
            "Are you afraid, your brothers and sisters",
            "are being brainwashed in school?"
        ],
        [
            "Who is drugging us, and is using",
            "radiation on us to control our minds?"
        ],
        [
            "Who uses psycho surgery and reprograms",
            "peoples' whole personalities?"
        ],
        [
            "Of course you already know the",
            "answer to these questions."
        ],
        [
            "The government does this.",
            "They are behind everything!"
        ],
        [
            "So be prepared and take back control!",
            "Cause you are the ..."
        ]]
        

messages = {
        'motor':{
            'english':[ 
                'Your motor cortex is fried',
                'Topsy-turvy! Direction changed'
                ],
            'gibberish':[
                'Dein Motorcortex ist dahin',
                'Alles steht kopf!'
                ]
            },
        'frontal':{
            'english':[
                'Your frontal lobe is no more',
                'Reward system gome. Half points!'
                ],
            'gibberish':[
                'Der Frontallappen ist kaputt',
                'Nur halbe Belohnungen'
                ]
            },
        'parietal':{
            'english':[
                'Bye bye parietal lobe',
                'Spatial senses seem a little off'
                ],
            'gibberish':[
                'Tschuess Parietallappen',
                'Eben war noch alles groesser'
                ]
            },
        'occipital':{
            'english':[
                'Occipital lobe smashed',
                'Night vision? Nope.'
                ],
            'gibberish':[
                'Lobus occipitalis erat',
                'Fiat lux!'
                ]
            },
        'temporal':{
            'english':[
                'Your temporal lobe..',
                'Wait, what was I saying?'
                ],
            'gibberish':[
                'Dein Temporallappen ist..',
                'Language system kaputt.'
                ]
            }
        }

class TextController(object):
    def __init__(self):
        pygame.font.init()
        filename = data.filepath('fonts/yoster.ttf')
        self.font_yoster = pygame.font.Font(filename, 18)
        filename = data.filepath('fonts/manaspc.ttf')
        self.font_mana = pygame.font.Font(filename, 18)
        self.font_big = pygame.font.Font(filename, 30)

    def render_text(self, text, font, color=TEXT_COL):
        if font == 'yoster':
            return self.font_mana.render(text, False, color)
        elif font == 'mana':
            return self.font_yoster.render(text, False, color)

    def render_title(self, text, color=TEXT_COL):
        return self.font_big.render(text, False, color)


text_controller = TextController()


def lin_tween(cur_time, start_value, change, duration):
    return (change * cur_time / duration + start_value)

def quad_in(cur_time, start_value, change, durationd):
    cur_time /= durationd;
    return change * cur_time * cur_time + start_value;

def quad_out(cur_time, start_value, change, durationd):
    cur_time /= durationd;
    return -change * cur_time * (cur_time-2) + start_value

def point_on_circle(angle, radius, centerx, centery):
    x = centerx + (radius * math.cos(angle))
    y = centery + (radius * math.sin(angle))
    return (x, y)

def random_on_circle(radius, centerx, centery):
    angle = random.random() * math.pi * 2
    # angle += math.pi
    return point_on_circle(angle, radius, centerx, centery)

def random_on_arc(radius, centerx, centery):
    """Top part of semicircle
    """
    angle = random.random() * math.pi + math.pi
    return point_on_circle(angle, radius, centerx, centery)

def point_on_ellipse(angle, rad_x, rad_y, centerx, centery):
    x = centerx + (rad_x * math.cos(angle))
    y = centery + (rad_y * math.sin(angle))
    return (x, y)
        

class Sequence(object):
    def __init__(self, name, frames, fps, one_shot=False, callback=None):
        self.name = name
        self.frames = frames
        self.sec_per_frame = 1.0 / fps
        self.is_one_shot = one_shot
        self.callback = callback

    def on_finished(self):
        if self.is_one_shot and self.callback != None:
            self.callback()

    def __getitem__(self, index):
        if not self.is_one_shot:
            index %= len(self.frames)
        else:
            if index > len(self.frames) - 1:
                index = len(self.frames) -1
        return self.frames[index]


class Animation(object):
    def __init__(self):
        self.sequences = {}
        self.cur_sequ = None
        self.cur_time = 0.0
        self.cur_frame_index = 0
        self.callback = None
        self.is_finished = False

    def add_loop(self, name, frames, fps):
        self.sequences[name] = Sequence(name, frames, fps)

    def add_oneshot(self, name, frames, fps, callback):
        self.sequences[name] = Sequence(
                name, frames, fps, True, callback
                )

    def update(self, dt):
        if self.cur_sequ == None:
            return
        last_index = len(self.cur_sequ.frames) -1
        self.cur_time += dt
        if self.cur_time >= self.cur_sequ.sec_per_frame:
            self.cur_time -= self.cur_sequ.sec_per_frame
            self.cur_frame_index += 1
        if self.cur_frame_index > last_index:
            if not self.is_finished: 
                self.is_finished = True
                self.cur_sequ.on_finished()

    def play(self, name, reset=False):
        if self.cur_sequ != self.sequences[name] or reset:
            self.cur_time = 0.0
            self.cur_sequ = self.sequences[name]
            self.cur_frame_index = 0
            self.is_finished = False
        return (self.cur_sequ[self.cur_frame_index])

    def switch_loop(self, name):
        self.cur_sequ = self.sequences[name]


class InputController(object):
    def __init__(self):
        self.keys_down = []
        self.keys_pressed = []
        self.keys_released = []

    def clear_key_events(self):
        self.keys_pressed = []
        self.keys_released = []

    def define(self, input_name, *keycodes):
        for keycode in keycodes:
            self.input_keys[keycode] = input_name

    def pressed(self, key):
        if key in self.keys_pressed:
            self.keys_pressed.remove(key)
            return True
        return False

    def check(self, keycode):
        return keycode in self.keys_down

    def on_key_down(self, key):
        self.keys_pressed.append(key)
        self.keys_down.append(key)

    def on_key_up(self, key):
        self.keys_released.append(key)
        self.keys_down.remove(key)

    def on_mouse_bt_down(self, pos, button):
        pass

    def on_mouse_bt_up(self, pos, button):
        pass


